@@ -1,7 +1,7 @@
 hypervolume_overlap_confidence <- function(path1, path2, CI = .95, cores = 1) {
   exists_cluster = TRUE
   if(cores > 1 & getDoParWorkers() == 1) {
-    # If no cluster is registered, create a new one based on use input
+    # If no cluster is registered, create a new one based on user input
     cl = makeCluster(cores)
     clusterEvalQ(cl, {
       library(hypervolume)
@@ -9,6 +9,17 @@ hypervolume_overlap_confidence <- function(path1, path2, CI = .95, cores = 1) {
     registerDoParallel(cl)
     exists_cluster = FALSE
   }
+  if(cores >1 & getDoParWorkers() > 1) {
+    print("Exsisting cluster registered to doParallel. The existing cluster will be used instead of user specified number of cores.")
+  }
+  
+  on.exit({
+    # If a cluster was created for this specific function call, close cluster and register sequential backend
+    if(!exists_cluster) {
+      stopCluster(cl)
+      registerDoSEQ()
+    }
+  })
   
   if(list.files(path1)[1] != "resample 1.rds" | list.files(path2)[1] != "resample 1.rds") {
     if(!exists_cluster) {
@@ -31,10 +42,6 @@ hypervolume_overlap_confidence <- function(path1, path2, CI = .95, cores = 1) {
     "distribution" = distribution
   )
   
-  if(!exists_cluster) {
-    stopCluster(cl)
-    registerDoSEQ()
-  }
   
   return(results)
 }
